/*
** STRING.C - C String handling functions.
**
** Date     Programmer     History
** -------- -------------- --------------------------------------------------
** 880727   P. Eriksson    Template constructed.
** 880728   P. Eriksson    Added strcmpi(), stricmp(), strdup(), strerror()
*/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <charvar.h>
#include <malloc.h>


#ifdef tolower
#undef tolower
#endif

#ifdef toupper
#undef toupper
#endif

/*
** compare string without regard to case.
*/
int strcmpi(String1, String2)
char *String1;
char *String2;
{
   int Difference;


   while (!(Difference = tolower(*String1) - tolower(*String2)) && *String1)
   {
      String1++;
      String2++;
   }

   return Difference;
}



/*
** compare string without regard to case.
*/
int stricmp(String1, String2)
char *String1;
char *String2;
{
   return strcmpi(String1, String2);
}



/*
** duplicate string.
*/
char *strdup(String)
char *String;
{
   char *CP;

   if ((CP = (char *) malloc(strlen(String)+1)) == NULL)
      return (char *) NULL;
   else
      return strcpy(CP, String);
}



/*
** get system error text corresponding to the current 'errno' value.
*/
char *strerror(String)
char *String;
{
   fortran void ERTXT$();
   charvar(1025) Message;
   static char ReturnString[1500];
   char *CP;


   ERTXT$( (short) errno, Message);
   if (String)
   {
      strcpy(ReturnString, String);
      strcat(ReturnString, ": ");
   }
   else
      *ReturnString = '\0';

   CP = ReturnString + strlen(ReturnString);
   getcvar(CP, Message);
   strcat(ReturnString, "\n");
   return ReturnString;
}



/*
** convert lowercase letters to uppercase.
*/
char *strupr(String)
char *String;
{
   char *CP;

   CP = String;
   while (*CP)
      *CP++ = toupper(*CP);

   return String;
}



/*
** convert uppercase letters to lowercase.
*/
char *strlwr(String)
char *String;
{
   char *CP;

   CP = String;
   while (*CP)
      *CP++ = tolower(*CP);

   return String;
}



/*
** Append SourceString to TargetString.
*/
char *strcat(TargetString, SourceString)
char *TargetString;
char *SourceString;
{
   register char *CP;

   CP = TargetString;
   while (*CP) ++CP;
   while (*CP++ = *SourceString++);
   return TargetString;
}



/*
** Search for first occurence of 'Character' in 'String'.
*/
char *strchr(String, Character)
char *String;
int Character;
{
   while (*String && *String != Character)
      ++String;

   return (*String ? String : (char *) NULL);
}



/*
** Find the length of the first substring in 'SourceString' of
** characters not in 'CharacterSet'.
*/
size_t strcspn(SourceString, CharacterSet)
char *SourceString;
char *CharacterSet;
{
   register char *CP;

   CP = SourceString;
   while (*CP && strchr(CharacterSet, *CP)) ++CP;
   return CP - SourceString;
}



/*
** Append 'Length' characters of 'SourceString' to 'TargetString'.
*/
char *strncat(TargetString, SourceString, Length)
char *TargetString;
char *SourceString;
int Length;
{
   register char *CP;

   CP = TargetString;
   while (*CP) ++CP;
   while ((*CP++ = *SourceString++) && Length-- > 0);
   return TargetString;
}


/*
** Reverse the order of the characters in the string.
*/
char *strrev(String)
char *String;
{
   register char *CP1, *CP2, Temp;

   CP1 = CP2 = String;
   while (*CP2) CP2++;
   CP2--;

   while (CP1 < CP2)
   {
      Temp = *CP1;
      *CP1 = *CP2;
      *CP2 = Temp;
      CP1++;
      CP2--;
   }

   return String;
}


/*
** copy SourceString to TargetString.
*/
char *strcpy(TargetString, SourceString)
char *TargetString;
char *SourceString;
{
   register char *CP;

   CP = TargetString;
   while (*CP++ = *SourceString++);
   return TargetString;
}



/*
** copy 'Length' characters of 'SourceString' to 'TargetString'.
*/
char *strncpy(TargetString, SourceString, Length)
char *TargetString;
char *SourceString;
int   Length;
{
   register char *CP;

   CP = TargetString;
   while (Length-- > 0 && (*CP++ = *SourceString++));
   if (Length > 0)
      while (Length-- > 0)
         *CP++ = '\0';
   return TargetString;
}



/*
** Compare strings.
*/
int strcmp(String1, String2)
register char *String1;
register char *String2;
{
   register int Difference;

   while (!(Difference = *String1 - *String2) && *String1++ && *String2++);
   return Difference;
}



/*
** Returns the length of a 'String'.
*/
size_t strlen(String)
char *String;
{
   register char *CP;

   CP = String;
   while (*CP++);
   return CP - 1 - String;
}

